--[[
Main class for the add-on.  Handles all run-time functionality.
--]]

uab.classes.main = uab.mixins.aceOO.Class(uab.mixins.aceEvent)

function uab.classes.main.prototype:init()
	uab.classes.main.super.prototype.init(self)

    self.secureheader = uab.classes.SecureHeader:GetHeader("Global")
    
    self.name = "uab.main"	
	self.activators = {}
	self.combatRegistrations = {}
	self.combatUnregistrations = {}

    self:RegisterEvent("PLAYER_DEAD")
    self:RegisterEvent("PLAYER_REGEN_ENABLED")
end

function uab.classes.main.prototype:PLAYER_DEAD()
	for _, activator in pairs(self.activators) do
	    activator:IAmDead()
	end
end

function uab.classes.main.prototype:PLAYER_REGEN_ENABLED()
    --[[
    When we leave combat, we register and unregister any frames that requested to register/unregister
	during combat.
    --]]
    
	self:DelayedFrameRegister()
end

function uab.classes.main.prototype:Enable()
    self:CreateClickActivators()
    self:DisableActivators()
    
    self.ccframesEnabled = true

    self:ConnectToUnitFrames()
	self:EnableActivators()

	-- Make sure that we show the correct layout and at the correct scale
    uabLayoutManager:LayoutChanged(uab.db.profile.layout)
    self:ScaleChanged()
    
    self:GetCfgDlg():GetActionsTab():ActivatorsChanged()
end

function uab.classes.main.prototype:CreateClickActivators()
    self.numactivators = 0
    
	for key, val in pairs(uab.db.profile.activators) do
	    self:CreateActivator(key, val)
	end
    
    uab:RefreshActivatorList()
    
    self:GetCfgDlg():GetActionsTab():ActivatorsChanged()
end

function uab.classes.main.prototype:GetCfgDlg()
    if not self.cfgdlg then
        self.cfgdlg = uab.classes.cfgdlg:new()
    end
    
    return self.cfgdlg
end

function uab.classes.main.prototype:ActivatorExists(name)
    if self.activators[name] then
        return true
    end
end

function uab.classes.main.prototype:AnyActivators(name)
    local any
    
    for name, activator in pairs(self.activators) do
        if name and activator then
            any = true
            break
        end
    end
    
    return any
end

function uab.classes.main.prototype:NewActivator(name)
    local activation = self:FindNextAvailableActivation()

    if not activation then
        uab.error("No available activation click combinations available.")
        return
    end
        
    local newactivator = {}

	newactivator.enabled = true
    newactivator.button = activation.button
    newactivator.prefixshift = activation.prefixshift
    newactivator.prefixctrl = activation.prefixctrl
    newactivator.prefixalt = activation.prefixalt
    
    newactivator.groups = {}
    for _, info in pairs(uab.constants.groupInfo) do
        newactivator.groups[info.name] = {}
        newactivator.groups[info.name].enabled = true
    end
    
    uab.db.profile.activators[name] = newactivator

	self:CreateActivator(name, uab.db.profile.activators[name])
    self.activators[name]:Enable()

	--[[
	Hook this activator to every frame that is registered
	--]]
    for frame in pairs(self.ccframes) do
        self.activators[name]:RegisterFrame(frame)
    end

    uab:RefreshActivatorList()
    
    self:GetCfgDlg():GetActionsTab():ActivatorsChanged()
end

function uab.classes.main.prototype:GetActivator(name)
    return self.activators[name]
end

function uab.classes.main.prototype:CopyActivatorTo(from, to)
    self.activators[from]:CopyToHostile(self.activators[to], "hostile")
    self.activators[from]:CopyTo(self.activators[to])
end

function uab.classes.main.prototype:ClearActivator(name)
	self.activators[name]:Disable()
	self.activators[name] = nil
end

function uab.classes.main.prototype:RemoveActivator(name)
    self:ClearActivator(name)

	uab.db.profile.activators[name] = nil
	
	uab:RefreshActivatorList()
    
    self:GetCfgDlg():GetActionsTab():ActivatorsChanged()
end

function uab.classes.main.prototype:AddActivatorKey(name, key)
	local states = self:DisableActivators()
	
    uab.db.profile.activators[name]["prefix"..key] = true
    self.activators[name]:SetActivationData()

	self:EnableActivators(states)
end

function uab.classes.main.prototype:RemoveActivatorKey(name, key)
	local states = self:DisableActivators()

    uab.db.profile.activators[name]["prefix"..key] = nil
    self.activators[name]:SetActivationData()

	self:EnableActivators(states)
end

function uab.classes.main.prototype:ClearActivatorKeys(name)
	local states = self:DisableActivators()

    uab.db.profile.activators[name].prefixalt = nil
    uab.db.profile.activators[name].prefixctrl = nil
    uab.db.profile.activators[name].prefixshift = nil
    self.activators[name]:SetActivationData()

	self:EnableActivators(states)
end

function uab.classes.main.prototype:SetActivatorButton(name, btn)
	local states = self:DisableActivators()

    uab.db.profile.activators[name].button = btn
    self.activators[name]:SetActivationData()

	self:EnableActivators(states)
end

function uab.classes.main.prototype:DisableActivators()
	local states = {}
	self:IterateActivators(
		function(activator, states)
		    states[activator] = activator:Enabled()
		    activator:Disable()
		end,
		states)

	return states
end

function uab.classes.main.prototype:EnableActivators(states)
	self:IterateActivators(
		function(activator, states)
			if not states or states[activator] then
				activator:Enable()
			end
		end,
		states)
end

function uab.classes.main.prototype:DisableActivator(name)
    local activator = self.activators[name]
    
    if activator and activator:Enabled() then
        activator:Disable()
    end
end

function uab.classes.main.prototype:EnableActivator(name)
    local activator = self.activators[name]
    
    if activator and not activator:Enabled() then
        activator:Enable()
    end
end

function uab.classes.main.prototype:CreateActivator(name, data)
    self.numactivators = self.numactivators + 1
	self.activators[name] = uab.classes.ClickActivator:new(self.numactivators, name, data)

	uabFrameGroup:NewActivator(self.activators[name])
end

function uab.classes.main.prototype:ConnectToUnitFrames()

    self:HookClickCastFrames()

	self:RegisterAllFrames()
end

function uab.classes.main.prototype:RegisterAllFrames()
    -- Register all frames that snuck in before we did =)
    for frame in pairs(self.ccframes) do
		self:RegisterFrame(frame)
    end
end

function uab.classes.main.prototype:UnregisterAllFrames()
    -- Unregister all the frames we currently have registered
    for frame in pairs(self.ccframes) do
		self:UnregisterFrame(frame)
    end
end

function uab.classes.main.prototype:CCFNewIndex(t, k, v)
	if self.ccframesEnabled and not self.ccframesNewIndex then
	    if v == nil then
	        self:UnregisterFrame(k)
	    else
	        self:RegisterFrame(k)
	    end
	end
	
    --[[
	If there was someone loaded before us, and they catch
    ClickCastFrames changes, then let them have the event
    and let them do the rawset.  First, they need to process
    the event, and second, if they process it, they will
    either do the rawset or pass it to the next one in line
    as we are doing here.
    --]]
    
    if self.ccframesNewIndex then
        self.ccframesNewIndex(t,k,v)
    else
        if v == nil then
            rawset(self.ccframes, k, nil)
        else
            rawset(self.ccframes, k, v)
        end
    end
end

function uab.classes.main.prototype:HookClickCastFrames()
	if not self.ccframesHooked then
	    self.ccframesHooked = true
	    
		ClickCastFrames = ClickCastFrames or {}
		self.ccframes = ClickCastFrames

	    local mt = getmetatable(ClickCastFrames)
	    if not mt then
	        mt = {}
	    end

	    self.ccframesNewIndex = mt.__newindex

        if self.ccframesNewIndex then
            --[[
            Another click-cast add-on is already loaded.  We
            should note this, and not attach ourselves.  Any
            further actions will be retained in case at some
            reload, the other click-cast add-on is not loaded
            but the actions will have no effect in the current
            session.
            --]]
            
            uab.print("Another click-casting add-on is already loaded.")
            uab.print("If you want to use UnitActionBars, please disable the other add-on.")
            uab.print("You may set up action sets, actions, and general settings, but those")
            uab.print("settings will not have any effect, as UnitActionBars will not attach")
            uab.print("to any frame to enable click-casting.")
        end
        
	    local newindex = function(t,k,v)
	        uab.main:CCFNewIndex(t, k, v)
	    end

	    mt.__newindex = newindex

	    ClickCastFrames = setmetatable(ClickCastFrames, mt)
	end
	
    local tbl = {
		PlayerFrame,
		PetFrame,
		PartyMemberFrame1,
		PartyMemberFrame2,
		PartyMemberFrame3,
		PartyMemberFrame4,
		PartyMemberFrame1PetFrame,
		PartyMemberFrame2PetFrame,
		PartyMemberFrame3PetFrame,
		PartyMemberFrame4PetFrame,
		TargetFrame,
		TargetofTargetFrame,
    }

    for i, frame in pairs(tbl) do
		if not ClickCastFrames[frame] then
            ClickCastFrames[frame] = true
        end
    end
end

function uab.classes.main.prototype:Disable()
    self:DisconnectFromUnitFrames()
	self:DisableActivators()
    self:ClearActivators()
end

function uab.classes.main.prototype:ClearActivators()
	for name, activator in pairs(self.activators) do
        self:ClearActivator(name)
	end
end

function uab.classes.main.prototype:DisconnectFromUnitFrames()

	-- stop us from registering anyone while disconnecting
    self:UnhookClickCastFrames()

    -- Unregister all frames that are currently registered
    for frame in pairs(self.ccframes) do
		self:UnregisterFrame(frame)
    end
end

function uab.classes.main.prototype:UnhookClickCastFrames()
	--[[
	We don't actually disconnect from the metatable, as someone may
	have connected after us.  Instead, we just indicate that we should
	not process any registrations now.  They will still be in the table
	when we are re-enabled, and will be registered at that time.
	--]]

	self.ccframesEnabled = nil
end

function uab.classes.main.prototype:RegisterFrame(frame)
    if InCombatLockdown() then
        self:DelayRegisterFrame(frame)
        return
    end

    if uabFrameGroup:GetFrameGroup(frame) then
        self:EnableClickCastFrame(frame)
    end
end

function uab.classes.main.prototype:EnableClickCastFrame(frame)
    uab.pdebug("Registering frame : "..frame:GetName())
    
	self:IterateActivators("RegisterFrame", frame)
end

function uab.classes.main.prototype:UnregisterFrame(frame)
    if InCombatLockdown() then
        self:DelayUnregisterFrame(frame)
        return
    end

    self:DisableClickCastFrame(frame)
end

function uab.classes.main.prototype:DisableClickCastFrame(frame)
	self:IterateActivators("UnregisterFrame", frame)
end

function uab.classes.main.prototype:DelayRegisterFrame(frame)
	self.combatRegistrations[frame] = true
end

function uab.classes.main.prototype:DelayUnregisterFrame(frame)
	self.combatUnregistrations[frame] = true
end

function uab.classes.main.prototype:DelayedFrameRegister()
    self:RegisterDelayedFrames()
    self:UnregisterDelayedFrames()
end

function uab.classes.main.prototype:RegisterDelayedFrames()
	for frame, _ in pairs(self.combatRegistrations) do
        uab.print("Registering queued frame "..frame:GetName())
		self:RegisterFrame(frame)
        self.combatRegistrations[frame] = nil
	end
end

function uab.classes.main.prototype:UnregisterDelayedFrames()
	for frame, _ in pairs(self.combatUnregistrations) do
        uab.print("Unregistering queued frame "..frame:GetName())
		self:UnregisterFrame(frame)
        self.combatUnregistrations[frame] = nil
	end
end

function uab.classes.main.prototype:ActionsChanged(activator)
    uabLayoutManager:LayoutSettingsChanged()
end

function uab.classes.main.prototype:SecureHeaderChanged(groupname)
	self:IterateActivators("SecureHeaderChanged", groupname)
end

function uab.classes.main.prototype:ScaleChanged(groupname)
	self:IterateActivators("ScaleChanged", groupname)
end

function uab.classes.main.prototype:PositionChanged(groupname)
	self:IterateActivators("PositionChanged", groupname)
end

function uab.classes.main.prototype:DirectCastChanged()
	self:IterateActivators("DirectCastChanged")
end

function uab.classes.main.prototype:StayOpenChanged()
	self:IterateActivators("StayOpenChanged")
end

function uab.classes.main.prototype:FastChangeUnitChanged()
    self.secureheader:FastChangeUnitChanged()
	self:IterateActivators("FastChangeUnitChanged")
end

function uab.classes.main.prototype:FrameGroupsChanged()
	--[[
	Because we may have disabled UAB on some frame groups, and
    enabled it on others, we should unregister and reregister
    all the frames.
	--]]
    self:UnregisterAllFrames()
	self:RegisterAllFrames()
end

function uab.classes.main.prototype:HighlightBorderChanged()
	self:IterateActivators("HighlightBorderChanged")
end

function uab.classes.main.prototype:ResetButtons()
	self:IterateActivators("ResetButtons")
end

function uab.classes.main.prototype:ResetHeaders()
    --[[
    Make sure the SecureStateHeader frame is shown, even though it may not
    be in a state that should be shown.  We must first associate the header
    with some visible unit frame, then show it, then re-associate it with
    the prior parent.
    --]]
    local curparent = self.secureheader.frame:GetParent()
    
    for frame in pairs(self.ccframes) do
        if frame:IsVisible() then
            uab.print("UAB Resetting Header using parent frame : '"..frame:GetName().."'.")
            uab.print("  Please manually re-associate the action buttons to the correct before using them")
            self.secureheader.frame:SetParent(frame)
            self.secureheader.frame:Show()
            break
        end
    end
end

function uab.classes.main.prototype:FindNextAvailableActivation()
    --[[
    Find the first click combination that is not already used by
    some action set.  Search for the combination in the following order:

    The modifiers are tried in the following order for each button:
        
    shift
    ctrl
    alt
    shift-ctrl
    shift-alt
    ctrl-alt
    ctrl-shift-alt
    
    The buttons are tried in the following order:
    
    RightButton
    LeftButton
    MiddleButton
    Button4
    Button5
    
    If none of these match, the button alone is tried in the following
    order:
    
    MiddleButton
    Button4
    Button5
    RightButton
    LeftButton

    If all of these are taken, an error is generated, and no action set
    can be created.    
    --]]
    
    for idx, entry in pairs(uab.classes.ActionsTab.activations) do
        if not self:AnyActivatorWithActivation(entry) then
            return entry
        end
    end
end
            
function uab.classes.main.prototype:AnyActivatorWithActivation(entry)
    for name, activator in pairs(self.activators) do
        local settings = uab.db.profile.activators[name]
        
        if settings.button == entry.button and
            settings.prefixshift == entry.prefixshift and
            settings.prefixctrl == entry.prefixctrl and
            settings.prefixalt == entry.prefixalt then
            return true
        end
    end
end

function uab.classes.main.prototype:IterateActivators(fcn, context1, context2, context3)
	for _, activator in pairs(self.activators) do
		local result

		if type(fcn) == "string" then
		    result = activator[fcn](activator, context1, context2, context3)
		else
			result = fcn(activator, context1, context2, context3)
		end

		if result then
		    return result
		end
	end
end

