uab.modules.uabStatus_MainBuffs = uabUnitStatus:NewModule("Main Buffs")
uab.modules.uabStatus_MainBuffs.aura = AceLibrary("SpecialEvents-Aura-2.0")
uab.modules.uabStatus_MainBuffs.defaultDB = {
    disabled = nil,
    intellect = {
        disabled = nil,
        priority = 50,
        color = { r = 0, g = 0, b = 1, a = 1, },
        classfilter = {
            druid = true,
            hunter = true,
            mage = true,
            paladin = true,
            priest = true,
            rogue = nil,
            shaman = true,
            warlock = true,
            warrior = nil,
        },
	},
    fort = {
        disabled = nil,
        priority = 50,
        color = { r = 0, g = 0, b = 1, a = 1, },
        classfilter = {
            druid = true,
            hunter = true,
            mage = true,
            paladin = true,
            priest = true,
            rogue = true,
            shaman = true,
            warlock = true,
            warrior = true,
        },
	},
    shadowprot = {
        disabled = nil,
        priority = 50,
        color = { r = 0, g = 0, b = 1, a = 1, },
        classfilter = {
            druid = true,
            hunter = true,
            mage = true,
            paladin = true,
            priest = true,
            rogue = true,
            shaman = true,
            warlock = true,
            warrior = true,
        },
	},
    spirit = {
        disabled = nil,
        priority = 50,
        color = { r = 0, g = 0, b = 1, a = 1, },
        classfilter = {
            druid = true,
            hunter = true,
            mage = true,
            paladin = true,
            priest = true,
            rogue = nil,
            shaman = true,
            warlock = true,
            warrior = nil,
        },
	},
    mark = {
        disabled = nil,
        priority = 50,
        color = { r = 0, g = 0, b = 1, a = 1, },
        classfilter = {
            druid = true,
            hunter = true,
            mage = true,
            paladin = true,
            priest = true,
            rogue = true,
            shaman = true,
            warlock = true,
            warrior = true,
        },
	},
}

function uab.modules.uabStatus_MainBuffs:OnInitialize()
    self.super.OnInitialize(self)

	self:RegisterUnitStatus("Buff: Arcane Intellect")
	self:RegisterUnitStatus("Buff: Fortitude")
	self:RegisterUnitStatus("Buff: Shadow Protection")
	self:RegisterUnitStatus("Buff: Divine Spirit")
	self:RegisterUnitStatus("Buff: Mark of the Wild")
end

function uab.modules.uabStatus_MainBuffs:OnEnable()
end

function uab.modules.uabStatus_MainBuffs:OnDisable()
end

function uab.modules.uabStatus_MainBuffs:IsStatusGained(name, unit, action)
    local class = string.lower(UnitClass(unit))
    local hasbuff
    
	if name == "Buff: Arcane Intellect" then
        if self.db.profile.intellect.classfilter[class] then
		    hasbuff = self.aura:UnitHasBuff(unit, "Arcane Intellect") or self.aura:UnitHasBuff(unit, "Arcane Brilliance")
        end
	elseif name == "Buff: Fortitude" then
        if self.db.profile.fort.classfilter[class] then
	        hasbuff = self.aura:UnitHasBuff(unit, "Power Word: Fortitude") or self.aura:UnitHasBuff(unit, "Prayer of Fortitude")
        end
	elseif name == "Buff: Shadow Protection" then
        if self.db.profile.shadowprot.classfilter[class] then
    	    hasbuff = self.aura:UnitHasBuff(unit, "Shadow Protection")
        end
	elseif name == "Buff: Divine Spirit" then
        if self.db.profile.spirit.classfilter[class] then
	        hasbuff = self.aura:UnitHasBuff(unit, "Divine Spirit") or self.aura:UnitHasBuff(unit, "Prayer of Spirit")
        end
	elseif name == "Buff: Mark of the Wild" then
        if self.db.profile.mark.classfilter[class] then
    	    hasbuff = self.aura:UnitHasBuff(unit, "Mark of the Wild") or self.aura:UnitHasBuff(unit, "Gift of the Wild")
        end
	end
    
    return hasbuff
end

function uab.modules.uabStatus_MainBuffs:OptionsForUnitStatus(name)
	if name == "Buff: Arcane Intellect" then
		return self.db.profile.intellect
	elseif name == "Buff: Fortitude" then
		return self.db.profile.fort
	elseif name == "Buff: Shadow Protection" then
		return self.db.profile.shadowprot
	elseif name == "Buff: Divine Spirit" then
		return self.db.profile.spirit
	elseif name == "Buff: Mark of the Wild" then
		return self.db.profile.mark
	end
end

function uab.modules.uabStatus_MainBuffs:CreateUnitStatusOptions()
    local args = {}

    args = {
        type = 'group',
        name = "Buffs",
        desc = "Configure the options for Buffs unit status values",
        args = {
            enabled = {
                type = 'toggle',
                name = "Enabled",
                desc = "When checked, enable the use of the Buffs unit status values",
                order = 1,
                get = function()
                    return not self.db.profile.disabled
                end,
                set = function(v)
                    self.db.profile.disabled = not v
                end,
            },

            spacing3 = {
                name = " ",
                type = 'header',
                order = 5,
            },

        },
    }

    args.args.intellect = self:CreateUnitStatusOption(self.db.profile.intellect, "Buff: Arcane Intellect", "Buff: Arcane Intellect")
    args.args.intellect.order = 10
    args.args.intellect.args.classfilter = self:CreateClassFilter(self.db.profile.intellect)
    args.args.intellect.args.classfilter.order = 40

    args.args.fort = self:CreateUnitStatusOption(self.db.profile.fort, "Buff: Fortitude", "Buff: Fortitude")
    args.args.fort.order = 20
    args.args.fort.args.classfilter = self:CreateClassFilter(self.db.profile.fort)
    args.args.fort.args.classfilter.order = 40

    args.args.spirit = self:CreateUnitStatusOption(self.db.profile.spirit, "Buff: Divine Spirit", "Buff: Divine Spirit")
    args.args.spirit.order = 30
    args.args.spirit.args.classfilter = self:CreateClassFilter(self.db.profile.spirit)
    args.args.spirit.args.classfilter.order = 40

    args.args.shadowprot = self:CreateUnitStatusOption(self.db.profile.shadowprot, "Buff: Shadow Protection", "Buff: Shadow Protection")
    args.args.shadowprot.order = 40
    args.args.shadowprot.args.classfilter = self:CreateClassFilter(self.db.profile.shadowprot)
    args.args.shadowprot.args.classfilter.order = 40

    args.args.mark = self:CreateUnitStatusOption(self.db.profile.mark, "Buff: Mark of the Wild", "Buff: Mark of the Wild")
    args.args.mark.order = 50
    args.args.mark.args.classfilter = self:CreateClassFilter(self.db.profile.mark)
    args.args.mark.args.classfilter.order = 40

    return args
end

function uab.modules.uabStatus_MainBuffs:CreateClassFilter(profile)
    if not profile.classfilter then
        profile.classfilter = {
            druid = true,
            hunter = true,
            mage = true,
            paladin = true,
            priest = true,
            rogue = true,
            shaman = true,
            warlock = true,
            warrior = true,
        }
    end
    
    local options = {
        type = 'group',
        name = "Class Filter",
        desc = "Set the class filter for this status",
        args = {
            druid = {
                type = 'toggle',
                name = "Druid",
                desc = "Check the status for a druid",
                order = 10,
                get = function() return profile.classfilter.druid end,
                set = function(v) profile.classfilter.druid = v end,
            },
            hunter = {
                type = 'toggle',
                name = "Hunter",
                desc = "Check the status for a hunter",
                order = 20,
                get = function() return profile.classfilter.hunter end,
                set = function(v) profile.classfilter.hunter = v end,
            },
            mage = {
                type = 'toggle',
                name = "Mage",
                desc = "Check the status for a mage",
                order = 30,
                get = function() return profile.classfilter.mage end,
                set = function(v) profile.classfilter.mage = v end,
            },
            paladin = {
                type = 'toggle',
                name = "Paladin",
                desc = "Check the status for a paladin",
                order = 40,
                get = function() return profile.classfilter.paladin end,
                set = function(v) profile.classfilter.paladin = v end,
            },
            priest = {
                type = 'toggle',
                name = "Priest",
                desc = "Check the status for a priest",
                order = 50,
                get = function() return profile.classfilter.priest end,
                set = function(v) profile.classfilter.priest = v end,
            },
            rogue = {
                type = 'toggle',
                name = "Rogue",
                desc = "Check the status for a rogue",
                order = 60,
                get = function() return profile.classfilter.rogue end,
                set = function(v) profile.classfilter.rogue = v end,
            },
            shaman = {
                type = 'toggle',
                name = "Shaman",
                desc = "Check the status for a shaman",
                order = 70,
                get = function() return profile.classfilter.shaman end,
                set = function(v) profile.classfilter.shaman = v end,
            },
            warlock = {
                type = 'toggle',
                name = "Warlock",
                desc = "Check the status for a warlock",
                order = 80,
                get = function() return profile.classfilter.warlock end,
                set = function(v) profile.classfilter.warlock = v end,
            },
            warrior = {
                type = 'toggle',
                name = "Warrior",
                desc = "Check the status for a warrior",
                order = 90,
                get = function() return profile.classfilter.warrior end,
                set = function(v) profile.classfilter.warrior = v end,
            },
        },
    }
    
    return options
end

